unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs,ExtCtrls,StdCtrls,
  http, uRequestRouter;

type
  TForm1 = class(TForm)
    ButtonConnect:TButton;
    ButtonDisconnect:TButton;
    Label22:TLabel;
    LabelConnectStatus:TLabel;
    ShapeConnection:TShape;
    procedure ButtonConnectClick(Sender:TObject);
    procedure ButtonDisconnectClick(Sender:TObject);
    procedure FormCloseQuery(Sender:TObject;var CanClose:boolean);
    procedure FormCreate(Sender:TObject);
    procedure FormDestroy(Sender:TObject);
  private
    fConnected: Boolean;
    fTCPHttpDaemon:TTCPHttpDaemon;
    imgFavicon: TMemoryStream;
    Procedure Connect;
    Procedure Disconnect;
    Procedure ShowConnectedState;
    Procedure MainRequest(Request,URI:string;Headers:TStringList; InputData,OutputData:TMemoryStream; var ResultCode: Integer);
    Procedure SomeRequest(Request,URI:string;Headers:TStringList; InputData,OutputData:TMemoryStream; var ResultCode: Integer);
    Procedure IconRequest(Request,URI:string;Headers:TStringList; InputData,OutputData:TMemoryStream; var ResultCode: Integer);
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

procedure TForm1.FormCreate(Sender:TObject);
begin
  RequestRouter:= TRequestRouter.Create;
  RequestRouter.Add('/',@Form1.MainRequest,true);
  RequestRouter.Add('/some',@Form1.SomeRequest);
  RequestRouter.Add('/favicon',@Form1.IconRequest);

  imgFavicon:= TMemoryStream.Create;
  imgFavicon.LoadFromFile('favicon.png');     //Icon shown in Browser Tab

end;

procedure TForm1.FormDestroy(Sender:TObject);
begin
  imgFavicon.Free;
  RequestRouter.Free;

end;

procedure TForm1.ButtonConnectClick(Sender:TObject);
begin
  Connect;
  ShowConnectedState;
end;

procedure TForm1.ButtonDisconnectClick(Sender:TObject);
begin
  Disconnect;
  ShowConnectedState;
end;

procedure TForm1.FormCloseQuery(Sender:TObject;var CanClose:boolean);
begin
  Disconnect;
end;

Procedure TForm1.Connect;
begin
  if fConnected then EXIT;

  //HTTPServer
  fTCPHttpDaemon:= TTCPHttpDaemon.Create('0.0.0.0','8080',15000); //IP-Adresse/Port/ClientTimeout
  fTCPHttpDaemon.OnRequest:= @RequestRouter.ProcessHttpRequest;

  fTCPHttpDaemon.Connect(fConnected);
  if not fConnected then fTCPHttpDaemon.Free;

end;

Procedure TForm1.Disconnect;
begin
  if not fConnected then EXIT;

  fConnected:= false;
  fTCPHttpDaemon.Terminate;

  sleep(200);

end;

Procedure TForm1.ShowConnectedState;
begin
  if fConnected then begin
    LabelConnectStatus.Caption:= 'Connected';
    ShapeConnection.Pen.Color:= clLime;
  end else begin
    LabelConnectStatus.Caption:= 'Disconnected';
    ShapeConnection.Pen.Color:= clGray;
  end//if

end;

Procedure TForm1.MainRequest(Request,URI:string;Headers:TStringList;InputData,
  OutputData:TMemoryStream;var ResultCode:Integer);
var
  site: string;
begin
  IF (request = 'GET') OR (request = 'POST') AND (URI = '/') THEN BEGIN
    Headers.Clear;
    Headers.Add('Content-type: Text/Html');

    site:=
      '<html>' +#13
    + '<head></head>' +#13
    + '<body>' +#13
    + 'Request Uri: ' + uri +#13
    + '<br>' +#13
    + 'ServerBeispiel: Aufruf von Form1.MainRequest' +#13
    + '</body>' +#13
    + '</html>';

    OutputData.Write(site[1],length(site));

    ResultCode:= 200;

  END;

end;

Procedure TForm1.SomeRequest(Request,URI:string;Headers:TStringList;InputData,
  OutputData:TMemoryStream;var ResultCode:Integer);
var site: string;
begin
  Headers.Clear;
  Headers.Add('Content-type: Text/Html');

  site:=
    '<html>' +#13
  + '<head></head>' +#13
  + '<body>' +#13
  + 'Request Uri: ' + uri +#13
  + '<br>' +#13
  + 'ServerBeispiel: Aufruf von Form1.SomeRequest' +#13
  + '</body>' +#13
  + '</html>';

  OutputData.Write(site[1],length(site));

  ResultCode:= 200;

end;

Procedure TForm1.IconRequest(Request,URI:string;Headers:TStringList;InputData,
  OutputData:TMemoryStream;var ResultCode:Integer);
begin
  //writeln('Request to:'+URI);

  If (request = 'GET') And (URI = '/favicon.ico') Then Begin
     ResultCode:= 200;
     OutputData.Write(imgFavicon.memory^, imgFavicon.Size); //Threadsave
     Headers.Clear;
     Headers.Add('Content-type: image/png');

  End;//If

end;

end.

