// Modified and extended by kupferstecher
// Original unit "http" file from Synapse library's http server example
// Synapse licence applies
unit http;

interface

uses
  Classes, blcksock, winsock, Synautil, SysUtils;

Type
  TRequestEvent = Procedure(Request,URI:string;Headers:TStringList;InputData,OutputData:TMemoryStream; var ResultCode: Integer) of object;


type
  TTCPHttpDaemon = class(TThread)
  public
    OnRequest: TRequestEvent;
  private
    LocalIP,LocalPort: string;
    ClientTimeout: Integer;
    Sock:TTCPBlockSocket;
  public
    Constructor Create(aLocalIP,aLocalPort:string;aClientTimeout:Integer);
    Destructor Destroy; override;
    Procedure Connect(out AConnected: Boolean);
    procedure Execute; override;
  end;

  TTCPHttpThrd = class(TThread)
  private
    Daemon: TTCPHttpDaemon; //saved on
    Sock:TTCPBlockSocket;
  public
    Headers: TStringList;
    InputData, OutputData: TMemoryStream;
    Constructor Create (hsock:tSocket;aDaemon:TTCPHttpDaemon);
    Destructor Destroy; override;
    procedure Execute; override;
    function ProcessHttpRequest(Request, URI: string): integer;
  end;

implementation

{ TTCPHttpDaemon }

Constructor TTCPHttpDaemon.Create(aLocalIP,aLocalPort:string;
  aClientTimeout:Integer);
begin

  sock:=TTCPBlockSocket.create;
  FreeOnTerminate:=true;
  LocalIP:= aLocalIP;
  LocalPort:= aLocalPort;
  ClientTimeout:= aClientTimeout;

  inherited create(true);
end;

Destructor TTCPHttpDaemon.Destroy;
begin
  Sock.free;
  inherited Destroy;
end;


Procedure TTCPHttpDaemon.Connect(out AConnected: Boolean);
begin
  AConnected:= true;
  Sock.CreateSocket;
  Sock.SetLinger(true,10);
  Sock.Bind(LocalIP,LocalPort);
  if Sock.LastError <> 0
  then begin AConnected:= false; EXIT; end;

  Sock.Listen;
  self.Start;
end;

procedure TTCPHttpDaemon.Execute;
var
  ClientSock:TSocket;
  vClientThread: TTCPHttpThrd;
begin

  WHILE NOT Terminated DO BEGIN
    if sock.canread(200) then begin
      ClientSock:= sock.accept;
      if sock.lastError=0
      then TTCPHttpThrd.create(ClientSock,self);
    end;
  END;

  Sock.AbortSocket;

end;

{ TTCPHttpThrd }

Constructor TTCPHttpThrd.Create(hsock:tSocket;aDaemon:TTCPHttpDaemon);
begin
  sock:=TTCPBlockSocket.create;
  Headers := TStringList.Create;
  InputData := TMemoryStream.Create;
  OutputData := TMemoryStream.Create;
  Sock.socket:=HSock;
  FreeOnTerminate:=true;
  Priority:=tpNormal;
  Daemon:= aDaemon;
  inherited create(false);
end;

Destructor TTCPHttpThrd.Destroy;
begin
  Sock.free;
  Headers.Free;
  InputData.Free;
  OutputData.Free;
  inherited Destroy;
end;

procedure TTCPHttpThrd.Execute;
var
  timeout: integer;
  s: string;
  method, uri, protocol: string;
  size: integer;
  x, n: integer;
  resultcode: integer;
  close: boolean;
begin
  timeout := Daemon.ClientTimeout;
  repeat
    //read request line
    s := sock.RecvString(timeout);
    if sock.lasterror <> 0 then
      Exit;
    if s = '' then
      Exit;
    method := fetch(s, ' ');
    if (s = '') or (method = '') then
      Exit;
    uri := fetch(s, ' ');
    if uri = '' then
      Exit;
    protocol := fetch(s, ' ');
    headers.Clear;
    size := -1;
    close := false;
    //read request headers
    if protocol <> '' then
    begin
      if pos('HTTP/', protocol) <> 1 then
        Exit;
      if pos('HTTP/1.1', protocol) <> 1 then
        close := true;
      repeat
        s := sock.RecvString(Timeout);
        if sock.lasterror <> 0 then
          Exit;
        if s <> '' then
          Headers.add(s);
        if Pos('CONTENT-LENGTH:', Uppercase(s)) = 1 then
          Size := StrToIntDef(SeparateRight(s, ' '), -1);
        if Pos('CONNECTION: CLOSE', Uppercase(s)) = 1 then
          close := true;
      until s = '';
    end;
    //recv document...
    InputData.Clear;
    if size >= 0 then
    begin
      InputData.SetSize(Size);
      x := Sock.RecvBufferEx(InputData.Memory, Size, Timeout);
      InputData.SetSize(x);
      if sock.lasterror <> 0 then
        Exit;
    end;
    OutputData.Clear;

    if assigned(Daemon.OnRequest)
    then Daemon.OnRequest(method,uri,headers,InputData,OutputData,ResultCode)
    else ResultCode := ProcessHttpRequest(method, uri);

    sock.SendString(protocol + ' ' + IntTostr(ResultCode) + CRLF);
    if protocol <> '' then
    begin
      headers.Add('Content-length: ' + IntTostr(OutputData.Size));
      if close then
        headers.Add('Connection: close');
      headers.Add('Date: ' + Rfc822DateTime(now));
      headers.Add('Server: Synapse HTTP server demo');
      headers.Add('');
      for n := 0 to headers.count - 1 do
        sock.sendstring(headers[n] + CRLF);
    end;
    if sock.lasterror <> 0 then
      Exit;
    Sock.SendBuffer(OutputData.Memory, OutputData.Size);
    if close then
      Break;
  until Sock.LastError <> 0;
end;

function TTCPHttpThrd.ProcessHttpRequest(Request, URI: string): integer;
var
  l: TStringlist;
begin
//sample of precessing HTTP request:
// InputData is uploaded document, headers is stringlist with request headers.
// Request is type of request and URI is URI of request
// OutputData is document with reply, headers is stringlist with reply headers.
// Result is result code
  result := 504;
  if request = 'GET' then
  begin
    headers.Clear;
    headers.Add('Content-type: Text/Html');
    l := TStringList.Create;
    try
      l.Add('<html>');
      l.Add('<head></head>');
      l.Add('<body>');
      l.Add('Request Uri: ' + uri);
      l.Add('<br>');
      l.Add('This document is generated by Synapse HTTP server demo!');
      l.Add('</body>');
      l.Add('</html>');
      l.SaveToStream(OutputData);
    finally
      l.free;
    end;
    Result := 200;
  end;
end;

end.
