// Autor: kupferstecher
//
// Achtung: Der Router stellt über eine Criticalsection zwar sicher, dass
//  nie zwei Anfragen sich überschneiden können, aber die Anfrage wird
//  trotzdem im Kontext des Verbindungs-Threads weitergegeben und nicht mit
//  dem Main-Thread synchronisiert.
unit uRequestRouter;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

Type
 TRouterRequestEvent = Procedure(Request,URI:string;Headers:TStringList; InputData,OutputData:TMemoryStream; var ResultCode: Integer) of object;

Type TRoute = class;


Type

{ TRequestRouter }

 TRequestRouter = class
  private
    EntryPointItem: TRoute;
    ItemList: TList;
    RouterCriticalSection: TRTLCriticalSection;
  public
    Constructor Create;
    Destructor Destroy;override;
    Procedure Add(ARoute:string; AOnRequest:TRouterRequestEvent; AEntryPoint: Boolean = false);
    Procedure ProcessHttpRequest(Request,URI:string;Headers:TStringList; InputData,OutputData:TMemoryStream; var ResultCode: Integer);
  private

end;

TRoute = class
  private
    OnRequest: TRouterRequestEvent;
    Route: string;
  public
    Constructor Create(ARoute:string; AOnRequest:TRouterRequestEvent);
end;

var
  RequestRouter: TRequestRouter;


IMPLEMENTATION



{ TRequestRouter }

Constructor TRequestRouter.Create;
begin
  ItemList:= TList.Create;

  InitCriticalSection(RouterCriticalSection);

end;

Destructor TRequestRouter.Destroy;
var vRoute: TRoute;
begin
  for pointer(vRoute) in ItemList
  do vRoute.Free;

  ItemList.Free;
  ItemList:= nil;

  if assigned(EntryPointItem) then EntryPointItem.Free;
  EntryPointItem:= nil;

  DoneCriticalSection(RouterCriticalSection);

end;

Procedure TRequestRouter.Add(ARoute:string;AOnRequest:TRouterRequestEvent;
  AEntryPoint:Boolean);
var vRoute: TRoute;
begin
  vRoute:= TRoute.Create(ARoute,AOnRequest);

  if AEntryPoint
  then EntryPointItem:= vRoute
  else ItemList.Add(vRoute);

end;

Procedure TRequestRouter.ProcessHttpRequest(Request,URI:string;
  Headers:TStringList;InputData,OutputData:TMemoryStream;var ResultCode:Integer
  );
var
  vRoute: TRoute;
  ii: Integer;
begin
  ENTERCRITICALSECTION(RouterCriticalSection);
    writeln('Router:'+URI);
    If (URI = '/') And assigned(EntryPointItem) Then Begin
      EntryPointItem.OnRequest(Request,URI,Headers,InputData,OutputData,ResultCode);

    End Else Begin

      for ii:= 0 to ItemList.Count-1 do begin
        vRoute:= TRoute(ItemList[ii]);
        if Pos(vRoute.Route,URI) =1 then begin
          vRoute.OnRequest(Request,URI,Headers,InputData,OutputData,ResultCode);
          BREAK;
        end;//if
      end;//do

    End;//If

  LEAVECRITICALSECTION(RouterCriticalSection);

end;

Constructor TRoute.Create(ARoute:string;AOnRequest:TRouterRequestEvent);
begin
  Route:= ARoute;
  OnRequest:= AOnRequest;
end;


end.

